import {
  IProjectInstanceModel,
  ProjectInstanceDbModel,
} from "../../features/projects/models/project_instance_database_model";
import { pipelineRealTimeService } from "../../features/_realtime/realtime_presentation";
import { App } from "../controllers/app";
import { CreateFolderUseCase } from "../usecases/create_folder_usecase";
import { SearchDataBaseModelUseCase } from "../usecases/search_database_model_usecase";

export class SetLastActivePipelineToRealTimeServiceScenario {
  call = async (): Promise<void> => {
    return (
      await new SearchDataBaseModelUseCase<IProjectInstanceModel>(ProjectInstanceDbModel).call({
        isActive: true,
      })
    ).fold(
      async (projectModel) => {
        if (projectModel.project === null) {
          return;
        }

        const projectPath = App.staticFilesStoreDir() + projectModel.rootDir + "/";
        await new CreateFolderUseCase().call(projectPath);
        pipelineRealTimeService.setPipelineDependency(
          projectModel.project.pipelines,
          projectPath,
          projectModel._id,
          projectModel.rootDir
        );
      },
      async (_e) => {
        console.log("not found active pipeline");
      }
    );
  };
}
