import { CallbackStrategyWithEmpty, CoreHttpController } from "../../core/controllers/http_controller";
import { Result } from "../../core/helpers/result";
import { EXEC_TYPE } from "../../core/models/exec_error_model";
import { ExecutorResult } from "../../core/models/executor_result";
import { IPipeline, IssueType, StackGenerateType } from "../../core/models/process_model";
import { StackService } from "../../core/services/stack_service";
import { TriggerType } from "../_triggers/models/trigger_database_model";

class NixStoreModel {}

const getNixStoreFolderCommand: IPipeline[] = [
  {
    process: {
      type: EXEC_TYPE.EXEC,
      command: `ls /nix/store`,
      isGenerating: true,
      isLocaleCode: false,
      issueType: IssueType.WARNING,
    },
    trigger: {
      type: TriggerType.FILE,
      value: ["context"],
    },
    env: null,
    stackGenerateType: StackGenerateType.SINGLETON,
  },
];
class GetNixStorePackagesUseCase extends CallbackStrategyWithEmpty {
  call = async () => {
    const stackService = new StackService(
      getNixStoreFolderCommand,
      "/Users/idontsudo/Desktop/testdeck-mocha-seed/server/build/test/"
    );
    stackService.call();

    const promise = new Promise((resolve, _reject) => {
      stackService.on((e) => {
        const iteration = e.firstElement();
        if (iteration.result instanceof ExecutorResult) {
          const nixPackage = iteration.result.data;
          resolve(nixPackage.split("\n").filter((e) => e.hasNoPattern(".drv")));
        } else {
          return "GetNixStorePackagesUseCase unknown Error";
        }
      });
    });
    return Result.ok(await promise);
  };
}

export class NixStoreManagerPresentation extends CoreHttpController<NixStoreModel> {
  constructor() {
    super({
      url: "nix_store_api",
    });
    super.get(new GetNixStorePackagesUseCase().call);
  }
}
