import { CallbackStrategyWithEmpty, ResponseBase } from "../../../core/controllers/http_controller";
import { Result } from "../../../core/helpers/result";
import { RobossemblerAssets } from "../../../core/models/robossembler_assets";
import { StaticFiles } from "../../../core/models/static_files";
import { ReadingJsonFileAndConvertingToInstanceClassScenario } from "../../../core/scenarios/read_file_and_json_to_plain_instance_class_scenario";
import { GetServerAddressUseCase } from "../../../core/usecases/get_server_address_usecase";
import { ProjectDBModel } from "../../_projects/models/project_database_model";

export class RobossemblerAssetsNetworkMapperScenario extends CallbackStrategyWithEmpty {
  async call(): ResponseBase {
    const projectDbModel = await ProjectDBModel.findOne({ isActive: true });
    if (projectDbModel === null) {
      return Result.error("is dont active projects");
    }
    const { rootDir } = projectDbModel;

    return new GetServerAddressUseCase().call().map(async (address) =>
      (
        await new ReadingJsonFileAndConvertingToInstanceClassScenario<RobossemblerAssets>(RobossemblerAssets).call(
          rootDir + StaticFiles.assets
        )
      ).map((model) => {
        return Result.ok(
          model.convertLocalPathsToServerPaths(
            `${address}/${
              rootDir.match(new RegExp(/[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}/gm))[0]
            }/assets`
          )
        );
      })
    );
  }
}
