import { App } from "../../../core/controllers/app";
import { CallbackStrategyWithIdQuery, ResponseBase } from "../../../core/controllers/http_controller";
import { Result } from "../../../core/helpers/result";
import { SetLastActivePipelineToRealTimeServiceScenario } from "../../../core/scenarios/set_active_pipeline_to_realtime_service_scenario";
import { CreateFolderUseCase } from "../../../core/usecases/create_folder_usecase";
import { ReadByIdDataBaseModelUseCase } from "../../../core/usecases/read_by_id_database_model_usecase";
import { UpdateDataBaseModelUseCase } from "../../../core/usecases/update_database_model_usecase";
import { MongoIdValidation } from "../../../core/validations/mongo_id_validation";
import { IProjectInstanceModel, ProjectInstanceDbModel } from "../models/project_instance_database_model";

export class SetActiveProjectScenario extends CallbackStrategyWithIdQuery {
  idValidationExpression = new MongoIdValidation();

  async call(id: string): ResponseBase {
    try {
      const result = await new ReadByIdDataBaseModelUseCase<IProjectInstanceModel>(ProjectInstanceDbModel).call(id);
      // id

      if (result.isFailure()) {
        return result.forward();
      }
      const model = result.value;

      return await (
        await new CreateFolderUseCase().call(App.staticFilesStoreDir() + model.rootDir)
      ).map(async () => {
        model.isActive = true;
        return (await new UpdateDataBaseModelUseCase(ProjectInstanceDbModel).call(model)).map(async (el) => {
          // TODO(IDONTSUDO): move it to a separate UseCase
          await ProjectInstanceDbModel.updateMany(
            { _id: { $ne: el._id }, isActive: { $eq: true } },
            { isActive: false }
          );
          await new SetLastActivePipelineToRealTimeServiceScenario().call();
          return Result.ok(`project ${id} is active`);
        });
      });
    } catch (error) {
      return Result.error("SetActiveProjectScenario error:" + String(error));
    }
  }
}
