import { CallbackStrategyWithFileUpload, ResponseBase } from "../../../core/controllers/http_controller";
import { IFile } from "../../../core/interfaces/file";
import { CreateFileUseCase } from "../../../core/usecases/create_file_usecase";
import { MongoIdValidation } from "../../../core/validations/mongo_id_validation";
import { IProjectInstanceModel } from "../models/project_instance_database_model";
import { ReadByIdDataBaseModelUseCase } from "../../../core/usecases/read_by_id_database_model_usecase";
import { ProjectDBModel } from "../../_projects/models/project_database_model";
import { ExecProcessUseCase } from "../../../core/usecases/exec_process_usecase";

export class UploadCadFileToProjectScenario extends CallbackStrategyWithFileUpload {
  checkingFileExpression: RegExp = new RegExp(".FCStd");
  idValidationExpression = new MongoIdValidation();

  async call(file: IFile, id: string): ResponseBase {
    return (await new ReadByIdDataBaseModelUseCase<IProjectInstanceModel>(ProjectDBModel).call(id)).map(
      async (databaseModel) =>
        (await new CreateFileUseCase().call(`${databaseModel.rootDir}/${file.name}`, file.data)).map(
          async () =>
            await new ExecProcessUseCase().call(
              `${databaseModel.rootDir}/`,
              '',
              `python3 $PYTHON_BLENDER --path '${databaseModel.rootDir}/assets/'`
            )
        )
    );
  }
}
